#!/usr/bin/perl

if ( $#ARGV != 2 ) {
    usage();
    exit;
}

$HTML_Header_File	= $ARGV[0];
$QA_Merged_File		= $ARGV[1];
$HTML_Tailer_File	= $ARGV[2];

%html_simple_replace = (
    "<SECTIONTITLE>"	=> "<H2>",
    "<\/SECTIONTITLE>"	=> "<\/H2>",
    "<SECTIONBODY>"	=> "",
    "</SECTIONBODY>"	=> "",
    "<QABody>"		=> "",
    "</QABody>"		=> "",
    "<Q>"		=> "<TABLE border=\"1\" align=\"center\" width=\"100%\">\n"
			 . "<TBODY>"
			 . "<TR>\n"
			 . " <TD>\n"
			 . "  <DL>",
    "</Q>"		=> "",
    "<QBODY>"		=> "   <DD>",
    "</QBODY>"		=> "   </DD>",
    "<ANS>"		=> "",
    "</ANS>"		=> "  </DL>\n"
			 . " </TD>\n"
			 . "</TR>\n"
			 . "</TBODY>\n"
			 . "</TABLE>",
    "<ATITLE>"		=> "   <DT>\n"
			 . "    A.",
    "</ATITLE>"		=> "   </DT>",
    "<SCRIPT>"		=> "<PRE>",
    "</SCRIPT>"		=> "</PRE>",
);



makehtml_header();
makehtml_body();
makehtml_tailer();

sub usage {
    print <<END;
make_html.perl usage:
    % perl make_html.perl <HTML header file> <qa merged file> <HTML tailer file>
END
;
}

sub makehtml_header {
    open( IN, $HTML_Header_File ) || die "$!";
    while( <IN> ) {
	$header_string_macro	= $header_string_macro . $_;
    }
    close( IN );
}




sub makehtml_body {
    my $still_header;
    my $curindex;
    my $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst;
    my @footnote, $footnoteindex;
    my $firstspace;

    $still_header	= 1;
    $footnoteindex	= 0;
    $firstspace		= 1;
    open( IN, $QA_Merged_File ) || die "$!";

    ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst )
	= localtime( time() );
    $Date	= sprintf( "%d/%02d/%02d %02d:%02d:%02d (%s)",
			   $year + 1900, $mon+1, $mday, $hour, $min, $sec,
			   (Sun,Mon,Tue,Wed,Thu,Fri,Sat)[$wday] );

    while( <IN> ) {
	if ( $still_header == 1 ) {

# HEADER TITLE, AUTHOR, DATE, COPYRIGHT, ʤɤ롣
	    if ( m/<TITLE:?\s*(.+)\s*>/ ) {
		$Title	= $1;
	    } elsif ( m/<AUTHOR:?\s*(.+)\s*>/ ) {
		$Author	= $1;
	    } elsif ( m/<COPYRIGHT:?\s*(.+)\s*>/ ) {
		$Copyright	= $1;
	    } elsif ( m/<HEADEREND>/ ) {
		$still_header	= 0;
		$result	= evaluate_string( $header_string_macro );
		print $result;
	    }

	} else {
# ʸ
	    my $pattern;

	    foreach $pattern ( keys %html_simple_replace ) {
		s/$pattern/$html_simple_replace{$pattern}/g;
	    }

	    if ( m/<QANDA:?\s*\.\/QAs\/(\d+)>/ ) {
		$curindex	= $1;
		next;
	    } 

	    if ( m/<FOOTNOTE:?\s*(.*)\s*>/ ) {
		$footnoteindex++;
		$footnote[$footnoteindex]	= $1;
		printf( "\[<A href=\"#%s-%d\"> %d<\/A>\]\n", $curindex, $footnoteindex, $footnoteindex );
		next;
	    }

	    s/<ABODY>/<DD>/g;

	    if ( m/<\/ABODY>/ ) {
		my $i;

		if ( $footnoteindex > 0 ) {
		    print "<HR width=\"50%\">\n";

		    for ( $i = 1; $i <= $footnoteindex; $i++ ) {
			print " <DT>[<A name=\"$curindex-$i\"> $i<\/A>]<\/DT>\n";
			print "  <DD>\n";
			print $footnote[$i];
			print "<\/DD>\n";
		    }

		    print " <\/DD>\n";
		    print "<\/DL>\n";
		}

		print "<DL>\n";

		$footnoteindex	= 0;
		next;
	    }

	    s|<QTITLE:?\s*(.*)\s*>|
		"<DT>\n" .
		" <A NAME=\"$curindex\">\n" .
		"  [\n" .
                "   <A href=\"#$curindex\">\n" .
                "    ֹ: $curindex\n" .
                "   </A>\n" .
                "  ]<BR>\n" .
		" </A>\n" .
		" Q. $1\n" .
		"</DT>"|eg;


            s|\[ֹ:?\s*(\d+)\s*\]|[<A href="#$1">ֹ: $1</A>]|g;

	    s|<ATITLE:?\s*(.*)\s*>|
		"<DT>\n" .
		" A. $1\n" .
		"</DT>"|eg;

#	    s|<REF:?\s*(\d+)\s*>|
#		"<A href\"#$1\">\n" .
#		"$1\n" .
#	        "</A>"|eg;

	    s|<URL:?\s*(.*)\s*>|
		"&lt;URL:<A href=\"$1\">\n" .
		"$1\n" .
		"</A>&gt;"|eg;

	    print;
	}
    }
    close( IN );
}

sub makehtml_tailer {
    my $tailer_string_macro;
    open( IN, $HTML_Tailer_File ) || die "$!";
    while( <IN> ) {
	$tailer_string_macro	= $tailer_string_macro . $_;
    }
    print evaluate_string( $tailer_string_macro );
    close( IN );
}

sub evaluate_string {
    my ( $val )	= @_;
    my $return;

    $val	=~ s|([\$]Author)| $Author |eg;
    $val	=~ s|([\$]Copyright)| $Copyright |eg;
    $val	=~ s|([\$]Date)| $Date |eg;
    $val	=~ s|([\$]Title)| $Title |eg;

    return $val;
}
